

#ifndef _BOOTSTRP_GENINFO_HXX
#define _BOOTSTRP_GENINFO_HXX

#include <tools/string.hxx>
#include <tools/list.hxx>

class GenericInformationList;

class GenericInformation : public ByteString	
{
friend class GenericInformationList;	
private:
	ByteString sValue;					
	ByteString sComment;

	GenericInformationList *pInfoList;	
	GenericInformationList *pParent;	

	void ListDeleted() { pParent = NULL; }	

public:
	GenericInformation( const ByteString &rKey, const ByteString &rValue,
			GenericInformationList *pParentList = NULL,
			GenericInformationList *pSubInfos = NULL );
	GenericInformation( const GenericInformation& rInf, BOOL bCopySubs = TRUE);

	~GenericInformation();

  ByteString &GetValue() { return sValue; }
  void SetValue( const ByteString &rValue ) { sValue = rValue; }

  ByteString &GetComment() { return sComment; }
  void SetComment( const ByteString &rComment ) { sComment = rComment; }

	BOOL InsertSubInfo( GenericInformation *pInfo );

  BOOL InsertSubInfo( const ByteString &rPathKey, const ByteString &rValue,
	    BOOL bSearchByPath = FALSE, BOOL bNewPath = FALSE);
	void RemoveSubInfo( GenericInformation *pInfo, BOOL bDelete = FALSE );

	GenericInformation *GetSubInfo( ByteString &rKey, BOOL bSearchByPath = FALSE,
		BOOL bCreatePath = FALSE );

	void SetSubList( GenericInformationList *pSubList )
		{ pInfoList = pSubList; }
	GenericInformationList *GetSubList() { return pInfoList; }
};

class GenericLockInformation : public GenericInformation
{
public:
GenericLockInformation( const ByteString &rKey, const ByteString &rValue,
			GenericInformationList *pParentList = NULL,
			GenericInformationList *pSubInfos = NULL )
  : GenericInformation( rKey, rValue, pParentList, pSubInfos),
	nLockKey( 0 ), aLockState( read ) {};

  enum LockState{ writeonly, read, readonly };

  BOOL SetWriteLock(UINT32 nKey = 0) { return ((read==aLockState) &&
		       (aLockState=writeonly, nLockKey=nKey, TRUE)); }

  BOOL ReleaseWriteLock(UINT32 nKey = 0) { return ((writeonly==aLockState) &&
			   (!nLockKey||nKey==nLockKey) &&
			   (aLockState=read, nLockKey=0, TRUE)); } 
  BOOL SetReadLock(UINT32 nKey = 0) { return ((read==aLockState) &&
		      (aLockState=readonly, nLockKey=nKey, TRUE)); }
  BOOL ReleaseReadLock(UINT32 nKey = 0) { return ((readonly==aLockState) &&
			  (!nLockKey||nKey==nLockKey) &&
			  (aLockState=read, nLockKey=0, TRUE)); } 

  LockState GetLockState() const { return aLockState; }
  BOOL IsWriteLocked() const { return (writeonly==aLockState); }
  BOOL IsReadLocked() const { return (readonly==aLockState); }
  BOOL IsNotLocked() const { return (read==aLockState); }
  BOOL IsLocker( UINT32 nKey ) { return (nKey==nLockKey || !nLockKey); }

  BOOL InsertSubInfo( GenericInformation *pInfo, UINT32 nKey = 0 ) {
	return ((writeonly==aLockState) &&
	 (!nLockKey || nKey==nLockKey) &&
	 (GenericInformation::InsertSubInfo( pInfo ), TRUE)); }

  BOOL InsertSubInfo( const ByteString &rPathKey, const ByteString &rValue, UINT32 nKey = 0,
	    BOOL bSearchByPath = FALSE, BOOL bNewPath = FALSE) {
	return ((writeonly==aLockState) &&
	 (!nLockKey || nKey==nLockKey) &&
	 (GenericInformation::InsertSubInfo( rPathKey, rValue, bSearchByPath, bNewPath ), TRUE)); }

  GenericInformation *GetSubInfo( ByteString &rKey, BOOL bSearchByPath = FALSE,
	  BOOL bCreatePath = FALSE, UINT32 nKey = 0 ) {
	if (writeonly==aLockState && nLockKey && nKey!=nLockKey )
	  return NULL;
	return GenericInformation::GetSubInfo(rKey, bSearchByPath, bCreatePath); }

private:

  LockState aLockState;
  UINT32    nLockKey;
};

DECLARE_LIST( GenericInformationList_Impl, GenericInformation * );

class GenericInformationList : public GenericInformationList_Impl
{
private:
	GenericInformation *pOwner;			

protected:

	ULONG InsertSorted( GenericInformation *pInfo, BOOL bOverwrite,
				ULONG nStart, ULONG nEnd );
	GenericInformation *Search( ULONG &rPos, ByteString sKey,
				ULONG nStart, ULONG nEnd );

public:
	GenericInformationList( GenericInformation *pParent = NULL );
	GenericInformationList(const GenericInformationList& rList, GenericInformation *pParent = NULL);
	~GenericInformationList();

	GenericInformation *GetInfo( ByteString &rKey, BOOL bSearchByPath = FALSE,
	     BOOL bCreatePath = FALSE );

	BOOL InsertInfo( GenericInformation *pInfo, BOOL bOverwrite = TRUE );

  BOOL InsertInfo( const ByteString &rPathKey, const ByteString &rValue,
	 BOOL bSearchByPath = FALSE, BOOL bNewPath = FALSE);
	void RemoveInfo( GenericInformation *pInfo, BOOL bDelete = FALSE );

	GenericInformation* SetOwner( GenericInformation *pNewOwner );

};

#endif

