
#ifndef _SV_PPDPARSER_HXX
#define _SV_PPDPARSER_HXX

#include <tools/string.hxx>
#include <tools/stream.hxx>
#include <tools/list.hxx>

class PPDParser;

enum PPDValueType { eInvocation, eQuoted, eSymbol, eString, eNo };

struct PPDValue
{
	PPDValueType	meType;
	String			maOption;
	String			maOptionTranslation;
	String			maValue;
	String			maValueTranslation;
};

DECLARE_LIST( PPDValueList, PPDValue* );

class PPDKey
{
	friend class PPDParser;

	String			maKey;
	PPDValueList	maValues;
	PPDValue*		mpDefaultValue;
	PPDValue*		mpQueryValue;

public:
	enum UIType { PickOne, PickMany, Boolean };
	enum SetupType { ExitServer, Prolog, DocumentSetup, PageSetup, JCLSetup, AnySetup };
private:

	BOOL			mbUIOption;
	String			maUITranslation;
	UIType			meUIType;
	int				mnOrderDependency;
	SetupType		meSetupType;
public:
	PPDKey( const String& rKey );
	~PPDKey();

	PPDValue*			insertValue( const String& rOption );
	int					countValues()
		{ return maValues.Count(); }

	PPDValue*			getValue( int n )
		{ return maValues.GetObject( n ); }
	PPDValue*			getValue( const String& rOption );
	PPDValue*			getDefaultValue() { return mpDefaultValue; }
	PPDValue*			getQueryValue() { return mpQueryValue; }

	const String&		getKey() { return maKey; }
	BOOL				isUIKey() { return mbUIOption; }
	const String&		getUITranslation() { return maUITranslation; }
	UIType				getUIType() { return meUIType; }
	SetupType			getSetupType() { return meSetupType; }
};

DECLARE_LIST( PPDKeyList, PPDKey* );

class PPDContext;

class PPDParser
{
	friend class PPDContext;
public:
	struct PPDConstraint
	{
		PPDKey*		mpKey1;
		PPDValue*	mpOption1;
		PPDKey*		mpKey2;
		PPDValue*	mpOption2;

		PPDConstraint() : mpKey1( NULL ), mpOption1( NULL ), mpKey2( NULL ), mpOption2( NULL ) {}
	};

	DECLARE_LIST( PPDConstraintList, PPDConstraint* )
	DECLARE_LIST( PPDParserList, PPDParser* )
	DECLARE_LIST( StringList, String* )

	static PPDParserList			aAllParsers;

	PPDKeyList						maKeys;
	PPDConstraintList				maConstraints;

	String							maPrinterName;
	String							maNickName;

	String							maFile;

	BOOL							mbColorDevice;
	ULONG							mnLanguageLevel;

	PPDValue*						mpDefaultImageableArea;
	PPDKey*							mpImageableAreas;

	PPDValue*						mpDefaultPaperDimension;
	PPDKey*							mpPaperDimensions;

	PPDValue*						mpDefaultInputSlot;
	PPDKey*							mpInputSlots;

	PPDValue*						mpDefaultResolution;
	PPDKey*							mpResolutions;

	PPDValue*						mpDefaultDuplexType;
	PPDKey*							mpDuplexTypes;

	PPDKey*							mpFontList;

	PPDParser( const String& rFile );
	~PPDParser();

	void parseOrderDependency( const String& rLine );
	void parseOpenUI( const String& rLine );
	void parseConstraint( const String& rLine );
	void parse( StringList& rLines );
public:
	static PPDParser* getParser( String aFile );
	static String getPPDPrinterName( const String& rFile );
	static void freeAll();

	const String&	getFilename() { return maFile; }

	PPDKey*			getKey( int n ) { return maKeys.GetObject( n ); }
	PPDKey*			getKey( const String& rKey );
	int				getKeys() { return maKeys.Count(); }
	BOOL			hasKey( PPDKey* );

	const PPDConstraintList& getConstraints() { return maConstraints; }

	const String&	getPrinterName() const
		{ return maPrinterName.Len() ? maPrinterName : maNickName; }
	const String&	getNickName() const
		{ return maNickName.Len() ? maNickName : maPrinterName; }

	BOOL			isColorDevice() { return mbColorDevice; }
	ULONG			getLanguageLevel() { return mnLanguageLevel; }

	const String&	getDefaultPaperDimension();
	const void		getDefaultPaperDimension( int& rWidth, int& rHeight )
		{ getPaperDimension( getDefaultPaperDimension(), rWidth, rHeight ); }
	BOOL getPaperDimension( const String& rPaperName,
				int& rWidth, int& rHeight );

	int 			getPaperDimensions()
		{ return mpPaperDimensions ? mpPaperDimensions->countValues() : 0; }
	const String&	getPaperDimension( int );
	const String&	getPaperDimensionCommand( int );
	const String&	getPaperDimensionCommand( const String & );

	const String&	matchPaper( int nWidth, int nHeight );

	BOOL getMargins( const String& rPaperName,
		 int &rLeft, int& rRight,
		 int &rUpper, int& rLower );

	const String&	getDefaultInputSlot();
	int				getInputSlots()
		{ return mpInputSlots ? mpInputSlots->countValues() : 0; }
	const String&	getSlot( int );
	const String&	getSlotCommand( int );
	const String&	getSlotCommand( const String& );

	void			getDefaultResolution( int& rXRes, int& rYRes );
	int				getResolutions();
	void			getResolution( int, int& rXRes, int& rYRes );
	const String&	getResolutionCommand( int nXRes, int nYRes );

	void			getResolutionFromString( const String&, int&, int& );

	const String&	getDefaultDuplexType();
	int				getDuplexTypes()
		{ return mpDuplexTypes ? mpDuplexTypes->countValues() : 0; }
	const String&	getDuplex( int );
	const String&	getDuplexCommand( int );
	const String&	getDuplexCommand( const String& );

	int				getFonts()
		{ return mpFontList ? mpFontList->countValues() : 0; }
	void			getFontAttributes( int,
						   String& rEncoding,
						   String& rCharset );
	void			getFontAttributes( const String&,
						   String& rEncoding,
						   String& rCharset );
	const String&	getFont( int );
};

class PPDContext
{
	PPDParser*			m_pParser;

public:
	struct PPDKeyValue
	{
		PPDKey*			m_pKey;
		PPDValue*		m_pCurrentValue;

		PPDKeyValue( PPDKey* pKey, PPDValue* pValue = NULL ) :
	m_pKey( pKey ), m_pCurrentValue( pValue ) {}
	};
	DECLARE_LIST( PPDKeyValueList, PPDKeyValue* )

	PPDKeyValueList		m_aCurrentValues;

	BOOL checkConstraints( PPDKeyValue*, PPDValue*, BOOL bDoReset = TRUE );
	BOOL resetValue( PPDKey*, BOOL bDefaultable = FALSE );
public:
	PPDContext( PPDParser* pParser = NULL );
	~PPDContext();

	void setParser( PPDParser* );
	PPDParser* getParser() { return m_pParser; }

	PPDValue* getValue( PPDKey* );
	PPDValue* setValue( PPDKey*, PPDValue*, BOOL bDontCareForConstraints = FALSE );

	int countValuesModified() { return m_aCurrentValues.Count(); }
	PPDKey* getModifiedKey( int n )
		{ return m_aCurrentValues.GetObject( n )->m_pKey; }

	BOOL checkConstraints( PPDKey*, PPDValue* );

	void*	getStreamableBuffer( ULONG& rBytes );
	void	rebuildFromStreamBuffer( void* pBuffer, ULONG nBytes );
};

#endif 
